// Draw a recursive Sierpinski Carpet, and its variants.
// Use nested geometric contexts.  Probably a bit easier
// than the Gasket, because everything happens on a square
// grid.  Use the '-' and '=' keys to change the number of
// recursive levels.

int levels = 1;

// A bunch of flags to keep track of which of the nine recursive
// calls we want to make.  You can get a variety of alternative
// fractials by flipping some of these values.
boolean[] codes = { 
  true, true, true, true, false, true, true, true, true };

void setup()
{
  size( 650, 650 );
  noStroke();
  fill( 0 );
}

void draw()
{
  background( 255 );
  translate( 25, 25 );
  scale( 600 );
  drawCarpet( levels );
}

void drawCarpet( int lev )
{
  if ( lev == 0 ) {
    // Base case: just draw a unit square.
    
    rect( 0, 0, 1, 1 );
  } else {
    // Recursive case: divide the square up into 3x3 subsquares.
    // Make recursive calls on some or all of them.
   
    for ( int row = 0; row < 3; ++row ) {
      for ( int col = 0; col < 3; ++col ) {
        int index = row*3 + col;
        if ( codes[index] ) {
          pushMatrix();
          translate( col/3.0, row/3.0 );
          scale( 1.0/3.0 );
          drawCarpet( lev - 1 );
          popMatrix();
        }
      }
    }
  }
}

void keyPressed()
{
  if( key == '-' ) {
    levels = max( 0, levels - 1 );
  } else if( key == '=' ) {
    ++levels;
  }
}
