// Draw a simple fractal with two-way branching of recursive
// circles.  Sort of inspired by Shiffman's Example 8.2 at
// http://natureofcode.com/book/chapter-8-fractals/.
// Embed each recursive call in its own geometric
// context, and use an explicit integer argument to track the
// number of recursive levels.

void setup()
{
  size( 500, 500 );
  fill( 128, 50 );
  stroke( 0 );
}

void makeDrawing( int levs )
{
  // Draw an ellipse at every level
  ellipse( 0, 0, 150, 150 );

  // Make recursive calls only if we haven't yet reached the base case.
  if ( levs > 0 ) {
    pushMatrix();
    rotate( map( mouseX, 0, width, 0, -radians( 60 ) ) );
    translate( 75, 0 );
    scale( map( mouseY, 0, height, 0, 1 ) );
    makeDrawing( levs - 1 );
    popMatrix();

    pushMatrix();
    rotate( PI + map( mouseX, 0, width, 0, radians( 60 ) ) );
    translate( 75, 0 );
    scale( map( mouseY, 0, height, 0, 1 ) );
    makeDrawing( levs - 1 );
    popMatrix();
  }
}

void draw()
{
  background( 255 );
  translate( 250, 250 );
  scale( 1.5 );
  makeDrawing( 10 );
}

