// Draw a Sierpinski Gasket using nested geometric contexts and
// an explicit argument to track the recursive level.  A good
// canonical demonstration of this technique, and probably a good
// starting point to use as an example for Assignment 06.

float hr3 = sqrt(3.0)/2.0;
int levels = 5;

void setup()
{ 
  // Make the sketch window exactly big enough to hold
  // an equilateral triangle.
  size( 500, int( 500 * hr3 ) );
}

void draw()
{
  background( 255 );
  translate( width/2, 0 );
  scale( width );
  noStroke();
  fill( 0 );

  drawTriangle( levels );
}

void drawTriangle( int lev )
{
  if ( lev == 0 ) {
  	// Base case: draw a single triangle of a fixed size
	// (1 unit across at the base, sqrt(3)/2 units tall)

    triangle( 0, 0, -0.5, hr3, 0.5, hr3 );
  } else {
    // Recursive case: set up three nested geometric
	// contexts, and make a single recursive call within 
	// each one.

    pushMatrix();
    scale( 0.5 );
    drawTriangle( lev - 1 );
    popMatrix();

    pushMatrix();
    translate( -0.25, 0.5*hr3 );
    scale( 0.5 );
    drawTriangle( lev - 1 );
    popMatrix();

    pushMatrix();
    translate( 0.25, 0.5*hr3 );
    scale( 0.5 );
    drawTriangle( lev - 1 );
    popMatrix();
  }
}

// A handy doodad for changing the recursive level interactively.
void keyPressed()
{
  if( key == '-' ) {
    levels = max( 0, levels - 1 );
  } else if( key == '=' ) {
    ++levels;
  }
}
