// Draw the Koch curve using nested geometric contexts.
// Do it in two levels, incorporating an intermediate reflection.

int levels = 5;
int lines;

void setup()
{
  size( 600, 400 );
}

void drawHalfKoch( int levs )
{
  pushMatrix();
  scale( 1.0/3.0 );
  drawKoch( levs - 1 );
  translate( 300, 0 );
  rotate( -radians( 60 ) );
  drawKoch( levs - 1 );
  popMatrix();
}

void drawKoch( int levs )
{
  if( levs == 0 ) {
    ++lines;
    line( 0, 0, 300, 0 );
  } else {
    // Draw the first half of the recursive curve
    drawHalfKoch( levs );
    
    // Now create a context that's reflected in a vertical
    // mirror and translated, and draw the same thing again.
    pushMatrix();
    translate( 150, 0 );
    scale( -1, 1 );
    translate( -150, 0 );
    drawHalfKoch( levs );
    popMatrix();
  }
}

void draw()
{
  background( 255 );
  noFill();
  stroke( 0 );
  strokeWeight( pow( 3, levels ) );
  
  lines = 0;
  
  translate( 50, 300 );
  scale( 500.0 / 300.0 );
  drawKoch( levels );
  // println( "Drew " + lines + " lines." );
  noLoop();
}

void keyPressed()
{
  if( key == '-' ) {
    levels = max( 0, levels - 1 );
    loop();
  } else if( key == '=' ) {
    ++levels;
    loop();
  }
}
