// Draw a Sierpinski Gasket by explicitly tracking the corners of
// a triangle.  Then, to make the recursive calls, you can compute
// the corners of the three smaller triangles you want to recurse on
// and continue with them.  Overall the structure is similar to
// ContextGasket.  Some students may find that this one's easier
// to think about (no confusing geometric contexts); on the other
// hand, it requires a bit more typing.  Could be a useful model for
// the "Pinwheel tiling" question on Assignment 06.

float hr3 = sqrt(3.0)/2.0;
int levels = 5;

void setup()
{ 
  size( 500, int( 500 * hr3 ) );
}

void draw()
{
  background( 255 );
  noStroke();
  fill( 0 );

  // Because the window size was chosen to fit an equilateral
  // triangle, the starting triangle can be derived directly from
  // the window size.
  drawTriangle( levels, width/2, 0, 0, height, width, height );
}

void drawTriangle(
  int levs, 
  float ax, float ay,
  float bx, float by,
  float cx, float cy )
{
  if( levs == 0 ) {
    // Base case -- no more recursive levels to draw.
    // Just draw a triangle.
    
    triangle( ax, ay, bx, by, cx, cy );
  } else {
    // Recursive case -- draw three recursive copies of
    // the gasket at the next level down.  Compute the
    // triangles bounding those recursive copies by computing
    // midpoints of the three points of the passed-in
    // triangle.
    
    float mabx = lerp( ax, bx, 0.5 );
    float maby = lerp( ay, by, 0.5 );
    float macx = lerp( ax, cx, 0.5 );
    float macy = lerp( ay, cy, 0.5 );
    float mbcx = lerp( bx, cx, 0.5 );
    float mbcy = lerp( by, cy, 0.5 );
    
    drawTriangle( levs - 1, ax, ay, mabx, maby, macx, macy );
    drawTriangle( levs - 1, mabx, maby, bx, by, mbcx, mbcy );
    drawTriangle( levs - 1, macx, macy, mbcx, mbcy, cx, cy );
  }
}

void keyPressed()
{
  if( key == '-' ) {
    levels = max( 0, levels - 1 );
  } else if( key == '=' ) {
    ++levels;
  }
}

