// Draw the Koch curve using explicit tracking of start and end
// points, as in CornerGasket.

int levels = 5;

void setup()
{
  size( 600, 400 );
}

void drawKoch( int lev, float x1, float y1, float x2, float y2 )
{
  if( lev == 0 ) {
    line( x1, y1, x2, y2 );
  } else {
    // Compute the locations of the three intermediate
    // points in the recursive curve, which can be used
    // as the start and end points for the recursive calls.
    float r = sqrt( 3.0 ) / 6.0;
    float ax = lerp( x1, x2, 0.3333 );
    float ay = lerp( y1, y2, 0.3333 );
    float bx = 0.5*(x1+x2) - r*(y1-y2);
    float by = 0.5*(y1+y2) - r*(x2-x1);
    float cx = lerp( x1, x2, 0.6666 );
    float cy = lerp( y1, y2, 0.6666 );
    
    drawKoch( lev - 1, x1, y1, ax, ay );
    drawKoch( lev - 1, ax, ay, bx, by );
    drawKoch( lev - 1, bx, by, cx, cy );
    drawKoch( lev - 1, cx, cy, x2, y2 );
  }
}

void draw()
{
  background( 255 );
  noFill();
  stroke( 0 );
  strokeWeight( 1 );
  
  drawKoch( levels, 50, 300, width-50, 300 );
  noLoop();
}

void keyPressed()
{
  if( key == '-' ) {
    levels = max( 0, levels - 1 );
    loop();
  } else if( key == '=' ) {
    ++levels;
    loop();
  }
}
