// A very simple model for creating a kind of fractal foam
// with circles.  Keep track of all the circles you've added
// already, and keep trying to find new points where a circle
// of the current radius still fits without intersecting any
// other circles.  If you do manage to add a circle, or fail
// enough times at the current radius, make the
// radius slightly smaller and keep going.

// More information (and examples):
//  http://john-art.com/gallery5.html
//  http://archive.bridgesmathart.org/2014/bridges2014-79.pdf
//  http://www.paulbourke.net/texture_colour/randomtile/

// All the circles we know about up to this point
float[] xs;
float[] ys;
float[] rs;

// The current radius of circle we're trying to place.
float radius;

// Do two given circles intersect?  I think this was
// covered briefly in CS 115x.
boolean checkIntersection(
float x1, float y1, float r1, 
float x2, float y2, float r2 )
{
  float d = dist( x1, y1, x2, y2 );
  return d <= (r1+r2);
}

boolean maybeAddCircle( float x, float y, float r )
{
  for ( int idx = 0; idx < xs.length; ++idx ) {
    if ( checkIntersection(
    x, y, r, xs[idx], ys[idx], rs[idx] ) ) {
      return false;
    }
  }

  xs = append( xs, x );
  ys = append( ys, y );
  rs = append( rs, r );
  return true;
}

void setup()
{
  size( 500, 500 );
  stroke( 0 );
  colorMode( HSB, 100 );
  initArrays();
}

void initArrays()
{
  xs = new float[0];
  ys = new float[0];
  rs = new float[0];
  radius = 80.0;
}

void draw()
{
  // Stop when the circles get small enough.
  if ( radius > 1 ) {
    // Try a thousand times to find a random circle location
    // that doesn't yet intersect other circles.  If you find 
    // it, draw it.  If you run out of tries, make the radius
    // slightly smaller.
    for ( int idx = 0; idx < 1000; ++idx ) {
      float x = random( width );
      float y = random( height );
      if ( maybeAddCircle( x, y, radius ) ) {
        float t = norm( radius, 80, 0 );

        fill( lerp( 0, 75, sq( t ) ), 100, 100 );
        ellipse( x, y, 2*radius, 2*radius );
        return;
      }
    }

    radius = radius * 0.99;
  }
}

void keyPressed()
{
  initArrays();
  background( 0, 0, 80 );
}

