// Using a method similar to the PerlinNoiseTiles example,
// create a simulation of an infinite scrolling faux 3D world
// using a coloured SVG hexagon as a building block.  Inspired
// by the classic arcade game Q*bert.

PShape hex;

float stride;

float ty = 0.0;

void setup()
{
  size( 500, 500 );
  hex = loadShape( "hex.svg" );
  stride = 0.75 * hex.height;
}

void draw()
{
  int parity = -int( ty / stride ) % 2;
  
  // The hardest part is to make sure the hexagons are
  // translated into the correct positions.  Hexagons tile
  // as nicely as squares, but the math is a bit messier.
  for( float y = ty%stride; y < height; y += stride ) {
    for( float x = -0.5*parity*hex.width; x < width; x += hex.width ) {
      pushMatrix();
      translate( x, y );
      translate( hex.width/2, hex.height/2 );
      // Here's the trick: you can make the grid look like a
      // random world by turning some of the hexagons upside
      // down.  Use the noise() function to make sure the 
      // decision is preserved from frame to frame.
      if( noise( x, y - ty ) > 0.5 ) {
        rotate( PI );
      }
      translate( -hex.width/2, -hex.height/2 );
      shape( hex, 0, 0 );
      popMatrix();
    }

    parity = 1 - parity;
  }
  
  ty -= 1.0;
}
