// Another classic example in the spirit of TenPrint -- draw
// an infinite field of what are sometimes called "Smith-Truchet
// tiles".  For a bit of extra work, colour in one half of the
// resulting diagram.

int tx;
int ty;

boolean do_fill;

void setup()
{
  size( 500, 500 );
  tx = 0;
  ty = 0;
  do_fill = false;
}

void draw()
{
  background( 255 );
  
  boolean base = (ty/50 + tx/50) % 2 == 0;
  // println( tx, ty, base );  
  for ( int y = ty%50 - 50; y < height; y += 50 ) {
    boolean rowbase = base;
    
    for ( int x = tx%50 - 50; x < width; x += 50 ) {
      pushMatrix();
      translate( x, y );
      drawTile( noise(x-tx, y-ty) < 0.5, base );
      base = !base;
      popMatrix();
    }
    base = !rowbase;
  }
}

void drawTile( boolean rotated, boolean cflip )
{
  int b1 = 50;
  int b2 = 240;
  if ( cflip != rotated ) {
    b1 = 240;
    b2 = 50;
  }
  noFill();

  clip( 0, 0, 50, 50 );
  noStroke();
  if( do_fill ) {
    fill( b1 );
  }
  rect( 0, 0, 50, 50 );

  if( do_fill ) {
    fill( b2 );
  }
  stroke( 0 );
  strokeWeight( 13.0 );
  if ( rotated ) {
    ellipse( 0, 0, 50, 50 );
    ellipse( 50, 50, 50, 50 );
  } else {
    ellipse( 50, 0, 50, 50 );
    ellipse( 0, 50, 50, 50 );
  }
  noClip();
}

void mouseDragged()
{
  tx += mouseX - pmouseX;
  ty += mouseY - pmouseY;
}

void keyPressed()
{
  if( key == ' ' ) {
    do_fill = !do_fill;
  }
}

